from flask import Blueprint, Flask, Config, url_for, request, redirect, session, jsonify, \
    make_response
from flask_oauthlib.client import OAuth, OAuthRemoteApp

from portal import oauth


def extract_user_info(me: dict) -> dict:
    return dict(
        uco=None,  # TODO: Need from gitlab or prompt the user
        name=me['name'],
        username=me['username'],
        email=me['email']
    )


def create_gitlab_app(oauth: OAuth) -> OAuthRemoteApp:
    base_url = oauth.app.config['GITLAB_URL']
    client_id = oauth.app.config['GITLAB_CLIENT_ID']
    client_secret = oauth.app.config['GITLAB_CLIENT_SECRET']

    return oauth.remote_app(
        'gitlab',
        base_url=base_url,
        request_token_url=None,
        access_token_url=base_url + "/oauth/token",
        authorize_url=base_url + "/oauth/authorize",
        access_token_method='POST',
        consumer_key=client_id,
        consumer_secret=client_secret
    )


gitlab = create_gitlab_app(oauth=oauth)
oauth = Blueprint('oauth', __name__, url_prefix='/oauth')


@oauth.route('/login', methods=['GET'])
def oauth_login():
    callback = url_for('oauth.oauth_authorized', _external=True, _scheme='https')
    return gitlab.authorize(callback=callback)


@oauth.route('/login/authorized', methods=['GET'])
def oauth_authorized():
    resp = gitlab.authorized_response()
    if resp is None:
        return 'Access denied: reason=%s error=%s' % (
            request.args['error'],
            request.args['error_description']
        )

    token = resp['access_token']
    session['gitlab_token'] = (token, '')
    me = gitlab.get('/api/v4/user')
    user_info = extract_user_info(me.data)
    data = dict(data=me.data, token=token)
    return jsonify(data)


@gitlab.tokengetter
def get_gitlab_oauth_token():
    token = session.get('gitlab_token')
    return token
